// IIFE за избягване на глобален scope
(function() {
    // Конфигурационни настройки
    const CONFIG = {
        debug: false, // Включване/изключване на конзолни логове
        markers: ['\u200B', '\u200C', '\u200D', '\uFEFF'],
        charMap: {
            // Кирилица към латиница
            'А': 'A', 'В': 'B', 'Е': 'E', 'К': 'K', 'М': 'M', 'Н': 'H', 
            'О': 'O', 'Р': 'P', 'С': 'C', 'Т': 'T', 'У': 'Y', 'Х': 'X',
            'а': 'a', 'е': 'e', 'о': 'o', 'р': 'p', 'с': 'c', 'у': 'y', 
            'х': 'x', 'З': '3', 'з': '3', 'і': 'i', 'І': 'I',
            // Латиница към кирилица
            'A': 'А', 'B': 'В', 'C': 'С', 'E': 'Е', 'H': 'Н', 'K': 'К', 
            'M': 'М', 'O': 'О', 'P': 'Р', 'T': 'Т', 'X': 'Х', 'Y': 'У',
            'a': 'а', 'e': 'е', 'o': 'о', 'p': 'р', 'c': 'с', 'y': 'у', 
            'x': 'х',
            // Цифри и символи
            '3': 'З', 'i': 'і', 'I': 'І'
        }
    };

    class TextObfuscator {
        constructor(config) {
            this.config = config;
            this.init();
        }

        init() {
            document.addEventListener('copy', this.handleCopy.bind(this));
            if (this.config.debug) {
                console.log('Text Obfuscator initialized');
            }
        }

        handleCopy(event) {
            const selection = document.getSelection();
            const selectedText = selection.toString();

            if (!selectedText.length) return;

            const obfuscatedText = this.obfuscateText(selectedText);
            const finalText = this.addAttribution(obfuscatedText);

            event.clipboardData.setData('text/plain', finalText);
            event.preventDefault();

            if (this.config.debug) {
                this.logDebugInfo(selectedText, obfuscatedText, finalText);
            }
        }

        obfuscateText(text) {
            return this.pipeline([
                this.replaceVisuallyIdenticalChars,
                this.randomizeCase,
                this.addInvisibleMarkers
            ], text);
        }

        // Помощна функция за последователно прилагане на трансформации
        pipeline(fns, initialValue) {
            return fns.reduce((value, fn) => fn.call(this, value), initialValue);
        }

        replaceVisuallyIdenticalChars(text) {
            return text.replace(/[A-Za-z0-9А-Яа-яІіЇї]/g, 
                char => this.config.charMap[char] || char
            );
        }

        addInvisibleMarkers(text) {
            let markerIndex = 0;
            return text.replace(/(\w+)/g, word => {
                const marker = this.config.markers[markerIndex];
                markerIndex = (markerIndex + 1) % this.config.markers.length;
                return word + marker;
            });
        }

        randomizeCase(text) {
            return text.split('').map(char => 
                Math.random() > 0.5 ? char.toUpperCase() : char.toLowerCase()
            ).join('');
        }

        addAttribution(text) {
            const siteName = document.querySelector('meta[property="og:site_name"]')?.content || 
                            document.title;
            
            const attribution = `\n\nИзточник: ${siteName}\n` +
                              `URL: ${window.location.href}\n` +
                              `© ${new Date().getFullYear()} Всички права запазени.`;
            
            return text + attribution;
        }

        logDebugInfo(original, obfuscated, final) {
            console.group('Text Obfuscation Debug');
            console.log('Original:', original);
            console.log('Obfuscated:', obfuscated);
            console.log('Final (with attribution):', final);
            console.groupEnd();
        }
    }

    // Инициализация
    try {
        window.textObfuscator = new TextObfuscator(CONFIG);
    } catch (error) {
        console.error('Failed to initialize Text Obfuscator:', error);
    }
})();